@@ -15,10 +15,7 @@ from pysal.model.spglm.utils import cache_readonly
 from .diagnostics import get_AIC, get_AICc, get_BIC, corr
 from .kernels import *
 from .summary import *
-
-fk = {'gaussian': fix_gauss, 'bisquare': fix_bisquare, 'exponential': fix_exp}
-ak = {'gaussian': adapt_gauss, 'bisquare': adapt_bisquare,
-      'exponential': adapt_exp}
+import multiprocessing as mp
 
 
 class GWR(GLM):
@@ -78,21 +75,12 @@ class GWR(GLM):
                     True to include intercept (default) in model and False to exclude
                     intercept.
 
-    dmat          : array
-                    n*n, distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
-    sorted_dmat   : array
-                    n*n, sorted distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
     spherical     : boolean
                     True for shperical coordinates (long-lat),
                     False for projected coordinates (defalut).
+    hat_matrix    : boolean
+                    True to store full n by n hat matrix,
+                    False to not store full hat matrix to minimize memory footprint (defalut).
 
     Attributes
     ----------
@@ -143,21 +131,13 @@ class GWR(GLM):
                     True to include intercept (default) in model and False to exclude
                     intercept
 
-    dmat          : array
-                    n*n, distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
-    sorted_dmat   : array
-                    n*n, sorted distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
     spherical     : boolean
                     True for shperical coordinates (long-lat),
                     False for projected coordinates (defalut).
+                    
+    hat_matrix    : boolean
+                    True to store full n by n hat matrix,
+                    False to not store full hat matrix to minimize memory footprint (defalut).
 
     n             : integer
                     number of observations
@@ -175,9 +155,6 @@ class GWR(GLM):
                     parameters passed into fit method to define estimation
                     routine
 
-    W             : array
-                    n*n, spatial weights matrix for weighting all
-                    observations from each calibration point
     points        : array-like
                     n*2, collection of n sets of (x,y) coordinates used for
                     calibration locations instead of all observations;
@@ -229,14 +206,14 @@ class GWR(GLM):
 
     def __init__(self, coords, y, X, bw, family=Gaussian(), offset=None,
                  sigma2_v1=True, kernel='bisquare', fixed=False, constant=True,
-                 dmat=None, sorted_dmat=None, spherical=False):
+                 spherical=False, hat_matrix=False):
         """
         Initialize class
         """
         GLM.__init__(self, y, X, family, constant=constant)
         self.constant = constant
         self.sigma2_v1 = sigma2_v1
-        self.coords = coords
+        self.coords = np.array(coords)
         self.bw = bw
         self.kernel = kernel
         self.fixed = fixed
@@ -250,31 +227,56 @@ class GWR(GLM):
         self.exog_scale = None
         self.exog_resid = None
         self.P = None
-        self.dmat = dmat
-        self.sorted_dmat = sorted_dmat
         self.spherical = spherical
-        self.W = self._build_W(fixed, kernel, coords, bw)
+        self.hat_matrix = hat_matrix
 
-    def _build_W(self, fixed, kernel, coords, bw, points=None):
-        if fixed:
-            try:
-                W = fk[kernel](coords, bw, points, self.dmat,
-                               self.sorted_dmat,
-                               spherical=self.spherical)
-            except BaseException:
-                raise  # TypeError('Unsupported kernel function  ', kernel)
+    def _build_wi(self, i, bw):
+
+        try:
+            wi = Kernel(i, self.coords, bw, fixed=self.fixed,
+                        function=self.kernel, points=self.points,
+                        spherical=self.spherical).kernel
+        except BaseException:
+            raise  # TypeError('Unsupported kernel function  ', kernel)
+
+        return wi
+
+    def _local_fit(self, i):
+        """
+        Local fitting at location i.
+        """
+        wi = self._build_wi(i, self.bw).reshape(-1, 1)  #local spatial weights
+
+        if isinstance(self.family, Gaussian):
+            betas, inv_xtx_xt = _compute_betas_gwr(self.y, self.X, wi)
+            predy = np.dot(self.X[i], betas)[0]
+            resid = self.y[i] - predy
+            influ = np.dot(self.X[i], inv_xtx_xt[:, i])
+            w = 1
+
+        elif isinstance(self.family, (Poisson, Binomial)):
+            rslt = iwls(self.y, self.X, self.family, self.offset, None,
+                        self.fit_params['ini_params'], self.fit_params['tol'],
+                        self.fit_params['max_iter'], wi=wi)
+            inv_xtx_xt = rslt[5]
+            w = rslt[3][i][0]
+            influ = np.dot(self.X[i], inv_xtx_xt[:, i]) * w
+            predy = rslt[1][i]
+            resid = self.y[i] - predy
+            betas = rslt[0]
+
+        if self.fit_params['lite']:
+            return influ, resid, predy, betas.reshape(-1)
         else:
-            try:
-                W = ak[kernel](coords, bw, points, self.dmat,
-                               self.sorted_dmat,
-                               spherical=self.spherical)
-            except BaseException:
-                raise  # TypeError('Unsupported kernel function  ', kernel)
+            Si = np.dot(self.X[i], inv_xtx_xt).reshape(-1)
+            tr_STS_i = np.sum(Si * Si * w * w)
+            CCT = np.diag(np.dot(inv_xtx_xt, inv_xtx_xt.T)).reshape(-1)
+            if not self.hat_matrix:
+                Si = None
+            return influ, resid, predy, betas.reshape(-1), w, Si, tr_STS_i, CCT
 
-        return W
-
-    def fit(self, ini_params=None, tol=1.0e-5, max_iter=20,
-            solve='iwls',searching = False):
+    def fit(self, ini_params=None, tol=1.0e-5, max_iter=20, solve='iwls',
+            lite=False, pool=None):
         """
         Method that fits a model with a particular estimation routine.
 
@@ -295,17 +297,18 @@ class GWR(GLM):
                         Technique to solve MLE equations.
                         Default is 'iwls', meaning iteratively (
                         re)weighted least squares.
-        searching     : bool, optional
+        lite          : bool, optional
                         Whether to estimate a lightweight GWR that
                         computes the minimum diagnostics needed for
                         bandwidth selection (could speed up
                         bandwidth selection for GWR) or to estimate
                         a full GWR. Default is False.
+        pool          : A multiprocessing Pool object to enable parallel fitting; default is None.
 
         Returns
         -------
                       :
-                        If searching=True, return a GWRResult
+                        If lite=False, return a GWRResult
                         instance; otherwise, return a GWRResultLite
                         instance.
 
@@ -314,53 +317,39 @@ class GWR(GLM):
         self.fit_params['tol'] = tol
         self.fit_params['max_iter'] = max_iter
         self.fit_params['solve'] = solve
+        self.fit_params['lite'] = lite
+
         if solve.lower() == 'iwls':
-            m = self.W.shape[0]
-
-            # In bandwidth selection, return GWRResultsLite
-            if searching:
-                resid = np.zeros((m, 1))
-                influ = np.zeros((m, 1))
-                for i in range(m):
-                    wi = self.W[i].reshape((-1, 1))
-                    if isinstance(self.family, Gaussian):
-                        betas, inv_xtx_xt = _compute_betas_gwr(
-                            self.y, self.X, wi)
-                        influ[i] = np.dot(self.X[i], inv_xtx_xt[:, i])
-                        predy = np.dot(self.X[i], betas)[0]
-                        resid[i] = self.y[i] - predy
-                    elif isinstance(self.family, (Poisson, Binomial)):
-                        rslt = iwls(self.y, self.X, self.family,
-                                    self.offset, None, ini_params, tol,
-                                    max_iter, wi=wi)
-                        inv_xtx_xt = rslt[5]
-                        influ[i] = np.dot(self.X[i], inv_xtx_xt[:, i]) * \
-                                   rslt[3][i][0]
-                        predy = rslt[1][i]
-                        resid[i] = self.y[i] - predy
-                return GWRResultsLite(self, resid, influ)
 
+            if self.points is None:
+                m = self.y.shape[0]
             else:
-                params = np.zeros((m, self.k))
-                predy = np.zeros((m, 1))
-                w = np.zeros((m, 1))
-                S = np.zeros((m, self.n))
-                CCT = np.zeros((m, self.k))
-                for i in range(m):
-                    wi = self.W[i].reshape((-1, 1))
-                    rslt = iwls(self.y, self.X, self.family,
-                                self.offset, None, ini_params, tol,
-                                max_iter, wi=wi)
-                    params[i, :] = rslt[0].T
-                    predy[i] = rslt[1][i]
-                    w[i] = rslt[3][i]
-                    S[i] = np.dot(self.X[i], rslt[5])
-                    # dont need unless f is explicitly passed for
-                    # prediction of non-sampled points
-                    #cf = rslt[5] - np.dot(rslt[5], f)
-                    #CCT[i] = np.diag(np.dot(cf, cf.T/rslt[3]))
-                    CCT[i] = np.diag(np.dot(rslt[5], rslt[5].T))
-                return GWRResults(self, params, predy, S, CCT, w)
+                m = self.points.shape[0]
+
+            if pool:
+                rslt = pool.map(self._local_fit,
+                                range(m))  #parallel using mp.Pool
+            else:
+                rslt = map(self._local_fit, range(m))  #sequential
+
+            rslt_list = list(zip(*rslt))
+            influ = np.array(rslt_list[0]).reshape(-1, 1)
+            resid = np.array(rslt_list[1]).reshape(-1, 1)
+            params = np.array(rslt_list[3])
+
+            if lite:
+                return GWRResultsLite(self, resid, influ, params)
+            else:
+                predy = np.array(rslt_list[2]).reshape(-1, 1)
+                w = np.array(rslt_list[-4]).reshape(-1, 1)
+                if self.hat_matrix:
+                    S = np.array(rslt_list[-3])
+                else:
+                    S = None
+                tr_STS = np.sum(np.array(rslt_list[-2]))
+                CCT = np.array(rslt_list[-1])
+                return GWRResults(self, params, predy, S, CCT, influ, tr_STS,
+                                  w)
 
     def predict(self, points, P, exog_scale=None, exog_resid=None,
                 fit_params={}):
@@ -404,12 +393,6 @@ class GWR(GLM):
             self.P = P
         else:
             self.P = P
-        self.W = self._build_W(
-            self.fixed,
-            self.kernel,
-            self.coords,
-            self.bw,
-            points)
         gwr = self.fit(**fit_params)
 
         return gwr
@@ -533,7 +516,10 @@ class GWRResults(GLMResults):
 
     R2                  : float
                           R-squared for the entire model (1- RSS/TSS)
-
+                          
+    adj_R2              : float
+                          adjusted R-squared for the entire model
+                          
     aic                 : float
                           Akaike information criterion
 
@@ -589,6 +575,14 @@ class GWRResults(GLMResults):
     pDev                : float
                           local percent of deviation accounted for; analogous to
                           r-squared for GLM's
+                          
+    D2                  : float
+                          percent deviance explained for GLM, equivaleng to R2 for
+                          Gaussian.
+                          
+    adj_D2              : float
+                          adjusted percent deviance explained, equivaleng to adjusted
+                          R2 for Gaussian.
 
     mu                  : array
                           n*, flat one dimensional array of predicted mean
@@ -604,17 +598,25 @@ class GWRResults(GLMResults):
                           unsampled points ()
     """
 
-    def __init__(self, model, params, predy, S, CCT, w=None):
+    def __init__(self, model, params, predy, S, CCT, influ, tr_STS=None,
+                 w=None):
         GLMResults.__init__(self, model, params, predy, w)
-        self.W = model.W
         self.offset = model.offset
         if w is not None:
             self.w = w
         self.predy = predy
         self.S = S
+        self.tr_STS = tr_STS
+        self.influ = influ
         self.CCT = self.cov_params(CCT, model.exog_scale)
         self._cache = {}
 
+    @cache_readonly
+    def W(self):
+        W = np.array(
+            [self.model._build_wi(i, self.model.bw) for i in range(self.n)])
+        return W
+
     @cache_readonly
     def resid_ss(self):
         if self.model.points is not None:
@@ -655,22 +657,14 @@ class GWRResults(GLMResults):
         """
         trace of S (hat) matrix
         """
-        return np.trace(self.S * self.w)
-
-    @cache_readonly
-    def tr_STS(self):
-        """
-        trace of STS matrix
-        """
-        return np.trace(np.dot(self.S.T * self.w, self.S * self.w))
+        return np.sum(self.influ)
 
     @cache_readonly
     def ENP(self):
         """
         effective number of parameters
 
-        Defualts to tr(s) as defined in yu et. al (2018) Inference in
-        Multiscale GWR
+        Defaults to tr(s) as defined in :cite:`yu:2019`
 
         but can alternatively be based on 2tr(s) - tr(STS)
 
@@ -693,7 +687,7 @@ class GWRResults(GLMResults):
         off = self.offset.reshape((-1, 1))
         arr_ybar = np.zeros(shape=(self.n, 1))
         for i in range(n):
-            w_i = np.reshape(np.array(self.W[i]), (-1, 1))
+            w_i = np.reshape(self.model._build_wi(i, self.model.bw), (-1, 1))
             sum_yw = np.sum(self.y.reshape((-1, 1)) * w_i)
             arr_ybar[i] = 1.0 * sum_yw / np.sum(w_i * off)
         return arr_ybar
@@ -715,8 +709,10 @@ class GWRResults(GLMResults):
             n = self.n
         TSS = np.zeros(shape=(n, 1))
         for i in range(n):
-            TSS[i] = np.sum(np.reshape(np.array(self.W[i]), (-1, 1)) *
-                            (self.y.reshape((-1, 1)) - self.y_bar[i])**2)
+            TSS[i] = np.sum(
+                np.reshape(self.model._build_wi(i, self.model.bw),
+                           (-1, 1)) * (self.y.reshape(
+                               (-1, 1)) - self.y_bar[i])**2)
         return TSS
 
     @cache_readonly
@@ -737,8 +733,9 @@ class GWRResults(GLMResults):
             resid = self.resid_response.reshape((-1, 1))
         RSS = np.zeros(shape=(n, 1))
         for i in range(n):
-            RSS[i] = np.sum(np.reshape(np.array(self.W[i]), (-1, 1))
-                            * resid**2)
+            RSS[i] = np.sum(
+                np.reshape(self.model._build_wi(i, self.model.bw),
+                           (-1, 1)) * resid**2)
         return RSS
 
     @cache_readonly
@@ -763,16 +760,16 @@ class GWRResults(GLMResults):
 
         if sigma2_v1 is True: only use n-tr(S) in denominator
 
-        Methods: p214, (9.6),
+        Methods: p214, (9.6) :cite:`fotheringham_geographically_2002`
         Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
         Geographically weighted regression: the analysis of spatially varying
         relationships.
 
-        and as defined  in Yu et. al. (2018) Inference in Multiscale GWR
+        and as defined in :cite:`yu:2019`
 
         if sigma2_v1 is False (v1v2): use n-2(tr(S)+tr(S'S)) in denominator
 
-        Methods: p55 (2.16)-(2.18)
+        Methods: p55 (2.16)-(2.18) :cite:`fotheringham_geographically_2002`
         Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
         Geographically weighted regression: the analysis of spatially varying
         relationships.
@@ -789,7 +786,7 @@ class GWRResults(GLMResults):
         """
         standardized residuals
 
-        Methods:  p215, (9.7)
+        Methods:  p215, (9.7) :cite:`fotheringham_geographically_2002`
         Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
         Geographically weighted regression: the analysis of spatially varying
         relationships.
@@ -802,26 +799,19 @@ class GWRResults(GLMResults):
         """
         standard errors of Betas
 
-        Methods:  p215, (2.15) and (2.21)
+        Methods:  p215, (2.15) and (2.21) :cite:`fotheringham_geographically_2002`
         Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
         Geographically weighted regression: the analysis of spatially varying
         relationships.
         """
         return np.sqrt(self.CCT)
 
-    @cache_readonly
-    def influ(self):
-        """
-        Influence: leading diagonal of S Matrix
-        """
-        return np.reshape(np.diag(self.S), (-1, 1))
-
     @cache_readonly
     def cooksD(self):
         """
         Influence: leading diagonal of S Matrix
 
-        Methods: p216, (9.11),
+        Methods: p216, (9.11) :cite:`fotheringham_geographically_2002`
         Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
         Geographically weighted regression: the analysis of spatially varying
         relationships.
@@ -839,7 +829,8 @@ class GWRResults(GLMResults):
                 'deviance not currently used for Gaussian')
         elif isinstance(self.family, Poisson):
             dev = np.sum(
-                2.0 * self.W * (y * np.log(y / (ybar * off)) - (y - ybar * off)), axis=1)
+                2.0 * self.W * (y * np.log(y / (ybar * off)) -
+                                (y - ybar * off)), axis=1)
         elif isinstance(self.family, Binomial):
             dev = self.family.deviance(self.y, self.y_bar, self.W, axis=1)
         return dev.reshape((-1, 1))
@@ -877,7 +868,7 @@ class GWRResults(GLMResults):
         testing. Includes corrected value for 90% (.1), 95% (.05), and 99%
         (.01) confidence levels. Correction comes from:
 
-        da Silva, A. R., & Fotheringham, A. S. (2015). The Multiple Testing Issue in
+        :cite:`Silva:2016` : da Silva, A. R., & Fotheringham, A. S. (2015). The Multiple Testing Issue in
         Geographically Weighted Regression. Geographical Analysis.
 
         """
@@ -888,7 +879,7 @@ class GWRResults(GLMResults):
 
     def critical_tval(self, alpha=None):
         """
-        Utility function to derive the critial t-value based on given alpha
+        Utility function to derive the critical t-value based on given alpha
         that are needed for hypothesis testing
 
         Parameters
@@ -927,7 +918,7 @@ class GWRResults(GLMResults):
 
         Parameters
         ----------
-        critical        : scalar
+        critical_t      : scalar
                           critical t-value to determine whether parameters are
                           statistically significant
 
@@ -982,28 +973,54 @@ class GWRResults(GLMResults):
     def pearson_chi2(self):
         return None
 
-    @cache_readonly
-    def null(self):
-        return None
-
     @cache_readonly
     def llnull(self):
         return None
 
     @cache_readonly
     def null_deviance(self):
-        return None
+        return self.family.deviance(self.y, self.null)
+
+    @cache_readonly
+    def global_deviance(self):
+        deviance = np.sum(self.family.resid_dev(self.y, self.mu)**2)
+        return deviance
+
+    @cache_readonly
+    def D2(self):
+        """
+        Percentage of deviance explanied. Equivalent to 1 - (deviance/null deviance)
+        """
+        D2 = 1.0 - (self.global_deviance / self.null_deviance)
+        return D2
 
     @cache_readonly
     def R2(self):
+        """
+        Global r-squared value for a Gaussian model.
+        """
         if isinstance(self.family, Gaussian):
-            TSS = np.sum((self.y.reshape((-1, 1)) -
-                          np.mean(self.y.reshape((-1, 1))))**2)
-            RSS = np.sum((self.y.reshape((-1, 1)) -
-                          self.predy.reshape((-1, 1)))**2)
-            return 1 - (RSS / TSS)
+            return self.D2
         else:
-            raise NotImplementedError('Only available for Gaussian GWR')
+            raise NotImplementedError('R2 only for Gaussian')
+
+    @cache_readonly
+    def adj_D2(self):
+        """
+        Adjusted percentage of deviance explanied.
+        """
+        adj_D2 = 1 - (1 - self.D2) * (self.n - 1) / (self.n - self.ENP - 1)
+        return adj_D2
+
+    @cache_readonly
+    def adj_R2(self):
+        """
+        Adjusted global r-squared for a Gaussian model.
+        """
+        if isinstance(self.family, Gaussian):
+            return self.adj_D2
+        else:
+            raise NotImplementedError('adjusted R2 only for Gaussian')
 
     @cache_readonly
     def aic(self):
@@ -1017,14 +1034,6 @@ class GWRResults(GLMResults):
     def bic(self):
         return get_BIC(self)
 
-    @cache_readonly
-    def D2(self):
-        return None
-
-    @cache_readonly
-    def adj_D2(self):
-        return None
-
     @cache_readonly
     def pseudoR2(self):
         return None
@@ -1057,7 +1066,7 @@ class GWRResults(GLMResults):
 
         Returns four arrays with the order and dimensions listed above where n
         is the number of locations used as calibrations points and p is the
-        nubmer of explanatory variables. Local correlation coefficient and local
+        number of explanatory variables. Local correlation coefficient and local
         VIF are not calculated for constant term.
 
         """
@@ -1080,25 +1089,25 @@ class GWRResults(GLMResults):
         vdp_pi = np.ndarray((nrow, nvar, nvar))
 
         for i in range(nrow):
-            wi = w[i]
+            wi = self.model._build_wi(i, self.model.bw)
             sw = np.sum(wi)
             wi = wi / sw
             tag = 0
 
             for j, k in jk:
-                corr_mat[i, tag] = corr(
-                    np.cov(x[:, j], x[:, k], aweights=wi))[0][1]
+                corr_mat[i, tag] = corr(np.cov(x[:, j], x[:, k],
+                                               aweights=wi))[0][1]
                 tag = tag + 1
 
             if self.model.constant:
                 corr_mati = corr(np.cov(x[:, 1:].T, aweights=wi))
-                vifs_mat[i, ] = np.diag(np.linalg.solve(
-                    corr_mati, np.identity((nvar - 1))))
+                vifs_mat[i, ] = np.diag(
+                    np.linalg.solve(corr_mati, np.identity((nvar - 1))))
 
             else:
                 corr_mati = corr(np.cov(x.T, aweights=wi))
-                vifs_mat[i, ] = np.diag(np.linalg.solve(
-                    corr_mati, np.identity((nvar))))
+                vifs_mat[i, ] = np.diag(
+                    np.linalg.solve(corr_mati, np.identity((nvar))))
 
             xw = x * wi.reshape((nrow, 1))
             sxw = np.sqrt(np.sum(xw**2, axis=0))
@@ -1173,12 +1182,10 @@ class GWRResults(GLMResults):
         for x in range(n_iters):
             temp_coords = np.random.permutation(self.model.coords)
             temp_sel.coords = temp_coords
-            temp_sel._build_dMat()
             temp_bw = temp_sel.search(**search_params)
-
-            temp_gwr.W = temp_gwr._build_W(fixed, kernel, temp_coords, temp_bw)
+            temp_gwr.bw = temp_bw
+            temp_gwr.coords = temp_coords
             temp_params = temp_gwr.fit(**fit_params).params
-
             temp_sd = np.std(temp_params, axis=0)
             SDs.append(temp_sd)
 
@@ -1239,13 +1246,14 @@ class GWRResultsLite(object):
 
     """
 
-    def __init__(self, model, resid, influ):
+    def __init__(self, model, resid, influ, params):
         self.y = model.y
         self.family = model.family
         self.n = model.n
         self.influ = influ
         self.resid_response = resid
         self.model = model
+        self.params = params
 
     @cache_readonly
     def tr_S(self):
@@ -1259,6 +1267,10 @@ class GWRResultsLite(object):
     def mu(self):
         return self.y - self.resid_response
 
+    @cache_readonly
+    def predy(self):
+        return self.y - self.resid_response
+
     @cache_readonly
     def resid_ss(self):
         u = self.resid_response.flatten()
@@ -1268,6 +1280,7 @@ class GWRResultsLite(object):
 class MGWR(GWR):
     """
     Multiscale GWR estimation and inference.
+    See :cite:`Fotheringham:2017` :cite:`yu:2019`.
 
     Parameters
     ----------
@@ -1314,20 +1327,13 @@ class MGWR(GWR):
                     True to include intercept (default) in model and False to exclude
                     intercept.
 
-    dmat          : array
-                    n*n, distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
-    sorted_dmat   : array
-                    n*n, sorted distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
     spherical     : boolean
-                    True for shperical coordinates (long-lat),
+                    True for spherical coordinates (long-lat),
                     False for projected coordinates (defalut).
+    hat_matrix    : boolean
+                    True for computing and storing covariate-specific
+                    hat matrices R (n,n,k) and model hat matrix S (n,n).
+                    False (default) for computing MGWR inference on the fly.
 
     Attributes
     ----------
@@ -1380,18 +1386,6 @@ class MGWR(GWR):
                     True to include intercept (default) in model and False to exclude
                     intercept.
 
-    dmat          : array
-                    n*n, distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
-    sorted_dmat   : array
-                    n*n, sorted distance matrix between calibration locations used
-                    to compute weight matrix. Defaults to None and is
-                    primarily for avoiding duplicate computation during
-                    bandwidth selection.
-
     spherical     : boolean
                     True for shperical coordinates (long-lat),
                     False for projected coordinates (defalut).
@@ -1445,19 +1439,21 @@ class MGWR(GWR):
     """
 
     def __init__(self, coords, y, X, selector, sigma2_v1=True,
-                 kernel='bisquare',
-                 fixed=False, constant=True, dmat=None,
-                 sorted_dmat=None, spherical=False):
+                 kernel='bisquare', fixed=False, constant=True,
+                 spherical=False, hat_matrix=False):
         """
         Initialize class
         """
         self.selector = selector
-        self.bw = self.selector.bw[0]
-        self.family = Gaussian()  # manually set since we only support Gassian MGWR for now
-        GWR.__init__(self, coords, y, X, self.bw, family=self.family,
+        self.bws = self.selector.bw[0]  #final set of bandwidth
+        self.bws_history = selector.bw[1]  #bws history in backfitting
+        self.bw_init = self.selector.bw_init  #initialization bandiwdth
+        self.family = Gaussian(
+        )  # manually set since we only support Gassian MGWR for now
+        GWR.__init__(self, coords, y, X, self.bw_init, family=self.family,
                      sigma2_v1=sigma2_v1, kernel=kernel, fixed=fixed,
-                     constant=constant, dmat=dmat, sorted_dmat=sorted_dmat,
-                     spherical=spherical)
+                     constant=constant, spherical=spherical,
+                     hat_matrix=hat_matrix)
         self.selector = selector
         self.sigma2_v1 = sigma2_v1
         self.points = None
@@ -1467,44 +1463,107 @@ class MGWR(GWR):
         self.exog_scale = None
         self_fit_params = None
 
-    # overwrite GWR method to handle multiple BW's
-    def _build_W(self, fixed, kernel, coords, bw, points=None):
-        Ws = []
-        for bw_i in bw:
-            if fixed:
-                try:
-                    W = fk[kernel](coords, bw_i, points, self.dmat,
-                                   self.sorted_dmat,
-                                   spherical=self.spherical)
-                except BaseException:
-                    raise  # TypeError('Unsupported kernel function  ', kernel)
-            else:
-                try:
-                    W = ak[kernel](coords, bw_i, points, self.dmat,
-                                   self.sorted_dmat,
-                                   spherical=self.spherical)
-                except BaseException:
-                    raise  # TypeError('Unsupported kernel function  ', kernel)
-            Ws.append(W)
-        return Ws
-
-    def fit(self):
+    def _chunk_compute_R(self, chunk_id=0):
         """
-        Method that extracts information from Sel_BW (selector) object and
-        prepares GAM estimation results for MGWRResults object.
-
+        Compute MGWR inference by chunks to reduce memory footprint.
         """
-        S = self.selector.S
-        R = self.selector.R
-        params = self.selector.params
-        predy = np.dot(S, self.y)
+        n = self.n
+        k = self.k
+        n_chunks = self.n_chunks
+        chunk_size = int(np.ceil(float(n / n_chunks)))
+        ENP_j = np.zeros(self.k)
         CCT = np.zeros((self.n, self.k))
-        for j in range(self.k):
-            C = np.dot(np.linalg.inv(np.diag(self.X[:, j])), R[:, :, j])
-            CCT[:, j] = np.diag(np.dot(C, C.T))
-        # manually set since we onlly support Gaussian MGWR for now
+
+        chunk_index = np.arange(n)[chunk_id * chunk_size:(chunk_id + 1) *
+                                   chunk_size]
+        init_pR = np.zeros((n, len(chunk_index)))
+        init_pR[chunk_index, :] = np.eye(len(chunk_index))
+        pR = np.zeros((n, len(chunk_index),
+                       k))  #partial R: n by chunk_size by k
+
+        for i in range(n):
+            wi = self._build_wi(i, self.bw_init).reshape(-1, 1)
+            xT = (self.X * wi).T
+            P = np.linalg.solve(xT.dot(self.X), xT).dot(init_pR).T
+            pR[i, :, :] = P * self.X[i]
+
+        err = init_pR - np.sum(pR, axis=2)  #n by chunk_size
+
+        for iter_i in range(self.bws_history.shape[0]):
+            for j in range(k):
+                pRj_old = pR[:, :, j] + err
+                Xj = self.X[:, j]
+                n_chunks_Aj = n_chunks
+                chunk_size_Aj = int(np.ceil(float(n / n_chunks_Aj)))
+                for chunk_Aj in range(n_chunks_Aj):
+                    chunk_index_Aj = np.arange(n)[chunk_Aj * chunk_size_Aj:(
+                        chunk_Aj + 1) * chunk_size_Aj]
+                    pAj = np.empty((len(chunk_index_Aj), n))
+                    for i in range(len(chunk_index_Aj)):
+                        index = chunk_index_Aj[i]
+                        wi = self._build_wi(index, self.bws_history[iter_i, j])
+                        xw = Xj * wi
+                        pAj[i, :] = Xj[index] / np.sum(xw * Xj) * xw
+                    pR[chunk_index_Aj, :, j] = pAj.dot(pRj_old)
+                err = pRj_old - pR[:, :, j]
+
+        for j in range(k):
+            CCT[:, j] += ((pR[:, :, j] / self.X[:, j].reshape(-1, 1))**2).sum(
+                axis=1)
+        for i in range(len(chunk_index)):
+            ENP_j += pR[chunk_index[i], i, :]
+
+        if self.hat_matrix:
+            return ENP_j, CCT, pR
+        return ENP_j, CCT
+
+    def fit(self, n_chunks=1, pool=None):
+        """
+        Compute MGWR inference by chunk to reduce memory footprint.
+        
+        Parameters
+        ----------
+
+        n_chunks      : integer, optional
+                        A number of chunks parameter to reduce memory usage. 
+                        e.g. n_chunks=2 should reduce overall memory usage by 2.
+        pool          : A multiprocessing Pool object to enable parallel fitting; default is None.
+                        
+        Returns
+        -------
+                      : MGWRResults
+        """
+        params = self.selector.params
+        predy = np.sum(self.X * params, axis=1).reshape(-1, 1)
+
+        try:
+            from tqdm.autonotebook import tqdm  #progress bar
+        except ImportError:
+
+            def tqdm(x, total=0,
+                     desc=''):  #otherwise, just passthrough the range
+                return x
+
+        if pool:
+            self.n_chunks = pool._processes * n_chunks
+            rslt = tqdm(
+                pool.imap(self._chunk_compute_R, range(self.n_chunks)),
+                total=self.n_chunks, desc='Inference')
+        else:
+            self.n_chunks = n_chunks
+            rslt = map(self._chunk_compute_R,
+                       tqdm(range(self.n_chunks), desc='Inference'))
+
+        rslt_list = list(zip(*rslt))
+        ENP_j = np.sum(np.array(rslt_list[0]), axis=0)
+        CCT = np.sum(np.array(rslt_list[1]), axis=0)
+
         w = np.ones(self.n)
-        return MGWRResults(self, params, predy, S, CCT, R, w)
+        if self.hat_matrix:
+            R = np.hstack(rslt_list[2])
+        else:
+            R = None
+        return MGWRResults(self, params, predy, CCT, ENP_j, w, R)
 
     def predict(self):
         '''
@@ -1529,10 +1588,10 @@ class MGWRResults(GWRResults):
                           n*1, predicted y values
 
     S                   : array
-                          n*n, hat matrix
+                          n*n, model hat matrix (if MGWR(hat_matrix=True))
 
     R                   : array
-                          n*n*k, partial hat matrices for each covariate
+                          n*n*k, covariate-specific hat matrices (if MGWR(hat_matrix=True))
 
     CCT                 : array
                           n*k, scaled variance-covariance matrix
@@ -1594,10 +1653,10 @@ class MGWRResults(GWRResults):
                           covariate (k)
 
     S                   : array
-                          n*n, hat matrix
+                          n*n, model hat matrix (if MGWR(hat_matrix=True))
 
     R                   : array
-                          n*n*k, partial hat matrices for each covariate
+                          n*n*k, covariate-specific hat matrices (if MGWR(hat_matrix=True))
 
     CCT                 : array
                           n*k, scaled variance-covariance matrix
@@ -1633,6 +1692,9 @@ class MGWRResults(GWRResults):
 
     R2                  : float
                           R-squared for the entire model (1- RSS/TSS)
+                          
+    adj_R2              : float
+                          adjusted R-squared for the entire model
 
     aic                 : float
                           Akaike information criterion
@@ -1675,16 +1737,29 @@ class MGWRResults(GWRResults):
 
     """
 
-    def __init__(self, model, params, predy, S, CCT, R, w):
+    def __init__(self, model, params, predy, CCT, ENP_j, w, R):
         """
         Initialize class
         """
-        GWRResults.__init__(self, model, params, predy, S, CCT, w)
+        self.ENP_j = ENP_j
         self.R = R
+        GWRResults.__init__(self, model, params, predy, None, CCT, None, w)
+        if model.hat_matrix:
+            self.S = np.sum(self.R, axis=2)
+        self.predy = predy
 
     @cache_readonly
-    def ENP_j(self):
-        return [np.trace(self.R[:, :, j]) for j in range(self.R.shape[2])]
+    def tr_S(self):
+        return np.sum(self.ENP_j)
+
+    @cache_readonly
+    def W(self):
+        Ws = []
+        for bw_j in self.model.bws:
+            W = np.array(
+                [self.model._build_wi(i, bw_j) for i in range(self.n)])
+            Ws.append(W)
+        return Ws
 
     @cache_readonly
     def adj_alpha_j(self):
@@ -1693,7 +1768,7 @@ class MGWRResults(GWRResults):
         testing. Includes corrected value for 90% (.1), 95% (.05), and 99%
         (.01) confidence levels. Correction comes from:
 
-        da Silva, A. R., & Fotheringham, A. S. (2015). The Multiple Testing Issue in
+        :cite:`Silva:2016` : da Silva, A. R., & Fotheringham, A. S. (2015). The Multiple Testing Issue in
         Geographically Weighted Regression. Geographical Analysis.
 
         """
@@ -1893,7 +1968,6 @@ class MGWRResults(GWRResults):
         for x in range(n_iters):
             temp_coords = np.random.permutation(self.model.coords)
             temp_sel.coords = temp_coords
-            temp_sel._build_dMat()
             temp_sel.search(**search_params)
             temp_params = temp_sel.params
             temp_sd = np.std(temp_params, axis=0)
