@@ -28,25 +28,13 @@ class TestGWRGaussian(unittest.TestCase):
         self.X = np.hstack([rural, pov, black])
         self.mgwr_X = np.hstack([fb, black, rural])
         self.BS_F = io.open(ps.examples.get_path('georgia_BS_F_listwise.csv'))
-        self.BS_NN = io.open(ps.examples.get_path('georgia_BS_NN_listwise.csv'))
+        self.BS_NN = io.open(
+            ps.examples.get_path('georgia_BS_NN_listwise.csv'))
         self.GS_F = io.open(ps.examples.get_path('georgia_GS_F_listwise.csv'))
-        self.GS_NN = io.open(ps.examples.get_path('georgia_GS_NN_listwise.csv'))
-        # self.BS_F = io.open(os.path.join(os.path.dirname(
-        #     __file__), 'georgia/georgia_BS_F_listwise.csv'))
-        # self.BS_NN = io.open(os.path.join(os.path.dirname(
-        #     __file__), 'georgia/georgia_BS_NN_listwise.csv'))
-        # self.GS_F = io.open(os.path.join(os.path.dirname(
-        #     __file__), 'georgia/georgia_GS_F_listwise.csv'))
-        # self.GS_NN = io.open(os.path.join(os.path.dirname(
-        #     __file__), 'georgia/georgia_GS_NN_listwise.csv'))
-        # MGWR_path = os.path.join(os.path.dirname(
-        #     __file__), 'georgia_mgwr_model_frame.csv')
-        # MGWRp_path = os.path.join(os.path.dirname(
-        #     __file__), 'georgia_mgwr_param_frame.csv')
-        # self.MGWR_n = pandas.read_csv(MGWR_path)
-        # self.MGWR_p = pandas.read_csv(MGWRp_path)
-        MGWR_path = os.path.join(os.path.dirname(__file__),
-                                 'georgia_mgwr_results.csv')
+        self.GS_NN = io.open(
+            ps.examples.get_path('georgia_GS_NN_listwise.csv'))
+        MGWR_path = os.path.join(
+            os.path.dirname(__file__), 'georgia_mgwr_results.csv')
         self.MGWR = pandas.read_csv(MGWR_path)
 
     def test_BS_F(self):
@@ -71,8 +59,8 @@ class TestGWRGaussian(unittest.TestCase):
 
         model = GWR(self.coords, self.y, self.X, bw=209267.689, fixed=True,
                     sigma2_v1=False)
-        rslt = model.fit()
 
+        rslt = model.fit()
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
@@ -141,13 +129,12 @@ class TestGWRGaussian(unittest.TestCase):
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
         CV = get_CV(rslt)
-
         corr2, vif2, cn2, vdp2 = rslt.local_collinearity()
-
         R2 = rslt.R2
 
-        np.testing.assert_allclose(adj_alpha, np.array([0.02034978,  0.01017489,
-                                                        0.0002035]), rtol=1e-04)
+        np.testing.assert_allclose(
+            adj_alpha, np.array([0.02034978, 0.01017489, 0.0002035]),
+            rtol=1e-04)
         self.assertAlmostEquals(critical_t, 2.6011011542649394)
         self.assertAlmostEquals(np.around(R2, 4), 0.5924)
         self.assertAlmostEquals(np.floor(AICc), 896.0)
@@ -208,8 +195,8 @@ class TestGWRGaussian(unittest.TestCase):
 
         model = GWR(self.coords, self.y, self.X, bw=87308.298,
                     kernel='gaussian', fixed=True, sigma2_v1=False)
-        rslt = model.fit()
 
+        rslt = model.fit()
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
@@ -258,16 +245,16 @@ class TestGWRGaussian(unittest.TestCase):
         inf = np.array(self.GS_NN.by_col(' influence')).reshape((-1, 1))
         cooksD = np.array(self.GS_NN.by_col(' CooksD')).reshape((-1, 1))
 
-        model = GWR(self.coords, self.y, self.X, bw=49.000,
-                    kernel='gaussian', fixed=False, sigma2_v1=False)
-        rslt = model.fit()
+        model = GWR(self.coords, self.y, self.X, bw=49.000, kernel='gaussian',
+                    fixed=False, sigma2_v1=False)
 
+        rslt = model.fit()
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
         CV = get_CV(rslt)
 
-        self.assertAlmostEquals(np.floor(AICc),  896.0)
+        self.assertAlmostEquals(np.floor(AICc), 896.0)
         self.assertAlmostEquals(np.floor(AIC), 894.0)
         self.assertAlmostEquals(np.floor(BIC), 922.0)
         self.assertAlmostEquals(np.around(CV, 2), 17.91)
@@ -294,12 +281,24 @@ class TestGWRGaussian(unittest.TestCase):
         std_y = (self.y - self.y.mean()) / self.y.std()
         std_X = (self.mgwr_X - self.mgwr_X.mean(axis=0)) / \
             self.mgwr_X.std(axis=0)
-        selector = Sel_BW(self.coords, std_y, std_X, multi=True,
-                          constant=True)
+        selector = Sel_BW(self.coords, std_y, std_X, multi=True, constant=True)
         selector.search(multi_bw_min=[2], multi_bw_max=[159])
         model = MGWR(self.coords, std_y, std_X, selector=selector,
                      constant=True)
+
         rslt = model.fit()
+        rslt_2 = model.fit(n_chunks=2)  #testing for n_chunks > 1
+        rslt_3 = model.fit(n_chunks=3)
+        rslt_20 = model.fit(n_chunks=20)
+
+        model_hat = MGWR(self.coords, std_y, std_X, selector=selector,
+                         constant=True, hat_matrix=True)
+        rslt_hat = model_hat.fit()
+        rslt_hat_2 = model_hat.fit(n_chunks=2)
+
+        np.testing.assert_allclose(rslt_hat.R, rslt_hat_2.R, atol=1e-07)
+        np.testing.assert_allclose(
+            rslt_hat.S.dot(std_y).flatten(), self.MGWR.predy, atol=1e-07)
 
         varnames = ['X0', 'X1', 'X2', 'X3']
 
@@ -309,32 +308,59 @@ class TestGWRGaussian(unittest.TestCase):
 
         np.testing.assert_allclose(rslt.predy.flatten(), self.MGWR.predy,
                                    atol=1e-07)
-        np.testing.assert_allclose(rslt.params,
-                                   self.MGWR[varnames].values, atol=1e-07)
-        np.testing.assert_allclose(rslt.bse,
-                                   self.MGWR[[s + "_bse" for s in varnames]].values, atol=1e-07)
-        np.testing.assert_allclose(rslt.tvalues,
-                                   self.MGWR[[s + "_tvalues" for s in varnames]].values, atol=1e-07)
+        np.testing.assert_allclose(rslt.params, self.MGWR[varnames].values,
+                                   atol=1e-07)
+        np.testing.assert_allclose(
+            rslt.bse, self.MGWR[[s + "_bse" for s in varnames]].values,
+            atol=1e-07)
+        np.testing.assert_allclose(
+            rslt_2.bse, self.MGWR[[s + "_bse" for s in varnames]].values,
+            atol=1e-07)
+        np.testing.assert_allclose(
+            rslt_3.bse, self.MGWR[[s + "_bse" for s in varnames]].values,
+            atol=1e-07)
+        np.testing.assert_allclose(
+            rslt_20.bse, self.MGWR[[s + "_bse" for s in varnames]].values,
+            atol=1e-07)
+        np.testing.assert_allclose(
+            rslt.tvalues, self.MGWR[[s + "_tvalues" for s in varnames]].values,
+            atol=1e-07)
         np.testing.assert_allclose(rslt.resid_response,
-                                   self.MGWR.resid_response,
-                                   atol=1e-04, rtol=1e-04)
+                                   self.MGWR.resid_response, atol=1e-04,
+                                   rtol=1e-04)
         np.testing.assert_almost_equal(rslt.resid_ss, 50.899379467870425)
         np.testing.assert_almost_equal(rslt.aicc, 297.12013812258783)
         np.testing.assert_almost_equal(rslt.ENP, 11.36825087269831)
-        np.testing.assert_allclose(rslt.ENP_j, [3.844671080264143, 3.513770805151652,
-                                                2.2580525278898254, 1.7517564593926895])
-        np.testing.assert_allclose(rslt.adj_alpha_j, np.array([[0.02601003, 0.01300501, 0.0002601 ],
-                                                               [0.02845945, 0.01422973, 0.00028459],
-                                                               [0.04428595, 0.02214297, 0.00044286],
-                                                               [0.05708556,
-                                                                0.02854278,
-                                                                0.00057086]]),  atol=1e-07)
-        np.testing.assert_allclose(rslt.critical_tval(),
-                                   np.array([2.51210749, 2.47888792,
-                                             2.31069113, 2.21000184]), atol=1e-07)
-        np.testing.assert_allclose(rslt.filter_tvals(),
-                                   self.MGWR[[s + "_filter_tvalues" for s in
-                                              varnames]].values, atol=1e-07)
+        np.testing.assert_allclose(rslt.ENP_j, [
+            3.844671080264143, 3.513770805151652, 2.2580525278898254,
+            1.7517564593926895
+        ])
+        np.testing.assert_allclose(rslt_2.ENP_j, [
+            3.844671080264143, 3.513770805151652, 2.2580525278898254,
+            1.7517564593926895
+        ])
+        np.testing.assert_allclose(rslt_3.ENP_j, [
+            3.844671080264143, 3.513770805151652, 2.2580525278898254,
+            1.7517564593926895
+        ])
+        np.testing.assert_allclose(rslt_20.ENP_j, [
+            3.844671080264143, 3.513770805151652, 2.2580525278898254,
+            1.7517564593926895
+        ])
+        np.testing.assert_allclose(
+            rslt.adj_alpha_j,
+            np.array([[0.02601003, 0.01300501, 0.0002601],
+                      [0.02845945, 0.01422973, 0.00028459],
+                      [0.04428595, 0.02214297, 0.00044286],
+                      [0.05708556, 0.02854278, 0.00057086]]), atol=1e-07)
+        np.testing.assert_allclose(
+            rslt.critical_tval(),
+            np.array([2.51210749, 2.47888792, 2.31069113, 2.21000184]),
+            atol=1e-07)
+        np.testing.assert_allclose(
+            rslt.filter_tvals(),
+            self.MGWR[[s + "_filter_tvalues" for s in varnames]].values,
+            atol=1e-07)
         np.testing.assert_allclose(rslt.local_collinearity()[0].flatten(),
                                    self.MGWR.local_collinearity, atol=1e-07)
 
@@ -344,80 +370,67 @@ class TestGWRGaussian(unittest.TestCase):
         test = index[-10:]
 
         X_test = self.X[test]
-        coords_test = list(coords[test])
+        coords_test = coords[test]
 
         model = GWR(self.coords, self.y, self.X, 93, family=Gaussian(),
                     fixed=False, kernel='bisquare', sigma2_v1=False)
         results = model.predict(coords_test, X_test)
 
-        params = np.array([22.77198, -0.10254,    -0.215093,   -0.01405,
-                           19.10531,    -0.094177,   -0.232529,   0.071913,
-                           19.743421,   -0.080447,   -0.30893,    0.083206,
-                           17.505759,   -0.078919,   -0.187955,   0.051719,
-                           27.747402,   -0.165335,   -0.208553,   0.004067,
-                           26.210627,   -0.138398,   -0.360514,   0.072199,
-                           18.034833,   -0.077047,   -0.260556,   0.084319,
-                           28.452802,   -0.163408,   -0.14097,    -0.063076,
-                           22.353095,   -0.103046,   -0.226654,   0.002992,
-                           18.220508,   -0.074034,   -0.309812,   0.108636]).reshape((10, 4))
+        params = np.array([
+            22.77198, -0.10254, -0.215093, -0.01405, 19.10531, -0.094177,
+            -0.232529, 0.071913, 19.743421, -0.080447, -0.30893, 0.083206,
+            17.505759, -0.078919, -0.187955, 0.051719, 27.747402, -0.165335,
+            -0.208553, 0.004067, 26.210627, -0.138398, -0.360514, 0.072199,
+            18.034833, -0.077047, -0.260556, 0.084319, 28.452802, -0.163408,
+            -0.14097, -0.063076, 22.353095, -0.103046, -0.226654, 0.002992,
+            18.220508, -0.074034, -0.309812, 0.108636
+        ]).reshape((10, 4))
         np.testing.assert_allclose(params, results.params, rtol=1e-03)
 
-        bse = np.array([2.080166,    0.021462,    0.102954,    0.049627,
-                        2.536355,    0.022111,    0.123857,    0.051917,
-                        1.967813,    0.019716,    0.102562,    0.054918,
-                        2.463219,    0.021745,    0.110297,    0.044189,
-                        1.556056,    0.019513,    0.12764,     0.040315,
-                        1.664108,    0.020114,    0.131208,    0.041613,
-                        2.5835,      0.021481,    0.113158,    0.047243,
-                        1.709483,    0.019752,    0.116944,    0.043636,
-                        1.958233,    0.020947,    0.09974,     0.049821,
-                        2.276849,    0.020122,    0.107867,    0.047842]).reshape((10, 4))
+        bse = np.array([
+            2.080166, 0.021462, 0.102954, 0.049627, 2.536355, 0.022111,
+            0.123857, 0.051917, 1.967813, 0.019716, 0.102562, 0.054918,
+            2.463219, 0.021745, 0.110297, 0.044189, 1.556056, 0.019513,
+            0.12764, 0.040315, 1.664108, 0.020114, 0.131208, 0.041613, 2.5835,
+            0.021481, 0.113158, 0.047243, 1.709483, 0.019752, 0.116944,
+            0.043636, 1.958233, 0.020947, 0.09974, 0.049821, 2.276849,
+            0.020122, 0.107867, 0.047842
+        ]).reshape((10, 4))
         np.testing.assert_allclose(bse, results.bse, rtol=1e-03)
 
-        tvalues = np.array([10.947193,   -4.777659,   -2.089223,   -0.283103,
-                            7.532584,    -4.259179,   -1.877395,   1.385161,
-                            10.033179,   -4.080362,   -3.012133,   1.515096,
-                            7.106862,    -3.629311,   -1.704079,   1.17042,
-                            17.831878,   -8.473156,   -1.633924,   0.100891,
-                            15.750552,   -6.880725,   -2.74765,    1.734978,
-                            6.980774,    -3.586757,   -2.302575,   1.784818,
-                            16.644095,   -8.273001,   -1.205451,   -1.445501,
-                            11.414933,   -4.919384,   -2.272458,   0.060064,
-                            8.00251, -3.679274,   -2.872176,   2.270738]).reshape((10, 4))
+        tvalues = np.array([
+            10.947193, -4.777659, -2.089223, -0.283103, 7.532584, -4.259179,
+            -1.877395, 1.385161, 10.033179, -4.080362, -3.012133, 1.515096,
+            7.106862, -3.629311, -1.704079, 1.17042, 17.831878, -8.473156,
+            -1.633924, 0.100891, 15.750552, -6.880725, -2.74765, 1.734978,
+            6.980774, -3.586757, -2.302575, 1.784818, 16.644095, -8.273001,
+            -1.205451, -1.445501, 11.414933, -4.919384, -2.272458, 0.060064,
+            8.00251, -3.679274, -2.872176, 2.270738
+        ]).reshape((10, 4))
         np.testing.assert_allclose(tvalues, results.tvalues, rtol=1e-03)
 
-        localR2 = np.array([[0.53068693],
-                            [0.59582647],
-                            [0.59700925],
-                            [0.45769954],
-                            [0.54634509],
-                            [0.5494828],
-                            [0.55159604],
-                            [0.55634237],
-                            [0.53903842],
+        localR2 = np.array([[0.53068693], [0.59582647], [0.59700925],
+                            [0.45769954], [0.54634509], [0.5494828],
+                            [0.55159604], [0.55634237], [0.53903842],
                             [0.55884954]])
         np.testing.assert_allclose(localR2, results.localR2, rtol=1e-05)
 
-        predictions = np.array([[10.51695514],
-                                [9.93321992],
-                                [8.92473026],
-                                [5.47350219],
-                                [8.61756585],
-                                [12.8141851],
-                                [5.55619405],
-                                [12.63004172],
-                                [8.70638418],
+        predictions = np.array([[10.51695514], [9.93321992], [8.92473026],
+                                [5.47350219], [8.61756585], [12.8141851],
+                                [5.55619405], [12.63004172], [8.70638418],
                                 [8.17582599]])
-        np.testing.assert_allclose(
-            predictions, results.predictions, rtol=1e-05)
+        np.testing.assert_allclose(predictions, results.predictions,
+                                   rtol=1e-05)
 
     def test_BS_NN_longlat(self):
-        GA_longlat = os.path.join(os.path.dirname(
-            __file__), 'ga_bs_nn_longlat_listwise.csv')
+        GA_longlat = os.path.join(
+            os.path.dirname(__file__), 'ga_bs_nn_longlat_listwise.csv')
         self.BS_NN_longlat = io.open(GA_longlat)
 
-        coords_longlat = list(zip(self.BS_NN_longlat.by_col(
-            ' x_coord'), self.BS_NN_longlat.by_col(' y_coord')))
+        coords_longlat = list(
+            zip(
+                self.BS_NN_longlat.by_col(' x_coord'),
+                self.BS_NN_longlat.by_col(' y_coord')))
         est_Int = self.BS_NN_longlat.by_col(' est_Intercept')
         se_Int = self.BS_NN_longlat.by_col(' se_Intercept')
         t_Int = self.BS_NN_longlat.by_col(' t_Intercept')
@@ -432,14 +445,14 @@ class TestGWRGaussian(unittest.TestCase):
         t_black = self.BS_NN_longlat.by_col(' t_PctBlack')
         yhat = self.BS_NN_longlat.by_col(' yhat')
         res = np.array(self.BS_NN_longlat.by_col(' residual'))
-        std_res = np.array(self.BS_NN_longlat.by_col(
-            ' std_residual')).reshape((-1, 1))
-        localR2 = np.array(self.BS_NN_longlat.by_col(
-            ' localR2')).reshape((-1, 1))
-        inf = np.array(self.BS_NN_longlat.by_col(
-            ' influence')).reshape((-1, 1))
-        cooksD = np.array(self.BS_NN_longlat.by_col(
-            ' CooksD')).reshape((-1, 1))
+        std_res = np.array(self.BS_NN_longlat.by_col(' std_residual')).reshape(
+            (-1, 1))
+        localR2 = np.array(self.BS_NN_longlat.by_col(' localR2')).reshape((-1,
+                                                                           1))
+        inf = np.array(self.BS_NN_longlat.by_col(' influence')).reshape((-1,
+                                                                         1))
+        cooksD = np.array(self.BS_NN_longlat.by_col(' CooksD')).reshape((-1,
+                                                                         1))
 
         model = GWR(coords_longlat, self.y, self.X, bw=90.000, fixed=False,
                     spherical=True, sigma2_v1=False)
@@ -449,7 +462,6 @@ class TestGWRGaussian(unittest.TestCase):
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
         CV = get_CV(rslt)
-
         R2 = rslt.R2
 
         self.assertAlmostEquals(np.around(R2, 4), 0.5921)
@@ -479,8 +491,8 @@ class TestGWRGaussian(unittest.TestCase):
 
 class TestGWRPoisson(unittest.TestCase):
     def setUp(self):
-        data_path = os.path.join(os.path.dirname(
-            __file__), 'tokyo/Tokyomortality.csv')
+        data_path = os.path.join(
+            os.path.dirname(__file__), 'tokyo/Tokyomortality.csv')
         data = io.open(data_path, mode='Ur')
         self.coords = list(
             zip(data.by_col('X_CENTROID'), data.by_col('Y_CENTROID')))
@@ -491,16 +503,22 @@ class TestGWRPoisson(unittest.TestCase):
         POP = np.array(data.by_col('POP65')).reshape((-1, 1))
         UNEMP = np.array(data.by_col('UNEMP')).reshape((-1, 1))
         self.X = np.hstack([OCC, OWN, POP, UNEMP])
-        self.BS_F = io.open(os.path.join(os.path.dirname(
-            __file__), 'tokyo/tokyo_BS_F_listwise.csv'))
-        self.BS_NN = io.open(os.path.join(os.path.dirname(
-            __file__), 'tokyo/tokyo_BS_NN_listwise.csv'))
-        self.GS_F = io.open(os.path.join(os.path.dirname(
-            __file__), 'tokyo/tokyo_GS_F_listwise.csv'))
-        self.GS_NN = io.open(os.path.join(os.path.dirname(
-            __file__), 'tokyo/tokyo_GS_NN_listwise.csv'))
-        self.BS_NN_OFF = io.open(os.path.join(os.path.dirname(
-            __file__), 'tokyo/tokyo_BS_NN_OFF_listwise.csv'))
+        self.BS_F = io.open(
+            os.path.join(
+                os.path.dirname(__file__), 'tokyo/tokyo_BS_F_listwise.csv'))
+        self.BS_NN = io.open(
+            os.path.join(
+                os.path.dirname(__file__), 'tokyo/tokyo_BS_NN_listwise.csv'))
+        self.GS_F = io.open(
+            os.path.join(
+                os.path.dirname(__file__), 'tokyo/tokyo_GS_F_listwise.csv'))
+        self.GS_NN = io.open(
+            os.path.join(
+                os.path.dirname(__file__), 'tokyo/tokyo_GS_NN_listwise.csv'))
+        self.BS_NN_OFF = io.open(
+            os.path.join(
+                os.path.dirname(__file__),
+                'tokyo/tokyo_BS_NN_OFF_listwise.csv'))
 
     def test_BS_F(self):
         est_Int = self.BS_F.by_col(' est_Intercept')
@@ -521,8 +539,9 @@ class TestGWRPoisson(unittest.TestCase):
         yhat = self.BS_F.by_col(' yhat')
         pdev = np.array(self.BS_F.by_col(' localpdev')).reshape((-1, 1))
 
-        model = GWR(self.coords, self.y, self.X, bw=26029.625, family=Poisson(),
-                    kernel='bisquare', fixed=True, sigma2_v1=False)
+        model = GWR(self.coords, self.y, self.X, bw=26029.625,
+                    family=Poisson(), kernel='bisquare', fixed=True,
+                    sigma2_v1=False)
         rslt = model.fit()
 
         AICc = get_AICc(rslt)
@@ -532,6 +551,7 @@ class TestGWRPoisson(unittest.TestCase):
         self.assertAlmostEquals(np.floor(AICc), 13294.0)
         self.assertAlmostEquals(np.floor(AIC), 13247.0)
         self.assertAlmostEquals(np.floor(BIC), 13485.0)
+
         np.testing.assert_allclose(est_Int, rslt.params[:, 0], rtol=1e-05)
         np.testing.assert_allclose(se_Int, rslt.bse[:, 0], rtol=1e-03)
         np.testing.assert_allclose(t_Int, rslt.tvalues[:, 0], rtol=1e-03)
@@ -576,10 +596,13 @@ class TestGWRPoisson(unittest.TestCase):
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
+        D2 = rslt.D2
 
         self.assertAlmostEquals(np.floor(AICc), 13285)
         self.assertAlmostEquals(np.floor(AIC), 13259.0)
         self.assertAlmostEquals(np.floor(BIC), 13442.0)
+        self.assertAlmostEquals(np.round(D2, 3), 0.747)
+
         np.testing.assert_allclose(est_Int, rslt.params[:, 0], rtol=1e-04)
         np.testing.assert_allclose(se_Int, rslt.bse[:, 0], rtol=1e-02)
         np.testing.assert_allclose(t_Int, rslt.tvalues[:, 0], rtol=1e-02)
@@ -617,39 +640,43 @@ class TestGWRPoisson(unittest.TestCase):
         yhat = self.BS_NN_OFF.by_col(' yhat')
         pdev = np.array(self.BS_NN_OFF.by_col(' localpdev')).reshape((-1, 1))
 
-        model = GWR(self.coords, self.y, self.X, bw=100, offset=self.off, family=Poisson(),
-                    kernel='bisquare', fixed=False, sigma2_v1=False)
+        model = GWR(self.coords, self.y, self.X, bw=100, offset=self.off,
+                    family=Poisson(), kernel='bisquare', fixed=False,
+                    sigma2_v1=False)
         rslt = model.fit()
 
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
+        D2 = rslt.D2
 
         self.assertAlmostEquals(np.floor(AICc), 367.0)
         self.assertAlmostEquals(np.floor(AIC), 361.0)
         self.assertAlmostEquals(np.floor(BIC), 451.0)
+        self.assertAlmostEquals(np.round(D2, 3), 0.676)
+
         np.testing.assert_allclose(est_Int, rslt.params[:, 0], rtol=1e-02,
                                    atol=1e-02)
-        np.testing.assert_allclose(
-            se_Int, rslt.bse[:, 0], rtol=1e-02, atol=1e-02)
+        np.testing.assert_allclose(se_Int, rslt.bse[:, 0], rtol=1e-02,
+                                   atol=1e-02)
         np.testing.assert_allclose(t_Int, rslt.tvalues[:, 0], rtol=1e-01,
                                    atol=1e-02)
         np.testing.assert_allclose(est_OCC, rslt.params[:, 1], rtol=1e-03,
                                    atol=1e-02)
-        np.testing.assert_allclose(
-            se_OCC, rslt.bse[:, 1], rtol=1e-02, atol=1e-02)
+        np.testing.assert_allclose(se_OCC, rslt.bse[:, 1], rtol=1e-02,
+                                   atol=1e-02)
         np.testing.assert_allclose(t_OCC, rslt.tvalues[:, 1], rtol=1e-01,
                                    atol=1e-02)
         np.testing.assert_allclose(est_OWN, rslt.params[:, 2], rtol=1e-04,
                                    atol=1e-02)
-        np.testing.assert_allclose(
-            se_OWN, rslt.bse[:, 2], rtol=1e-02, atol=1e-02)
+        np.testing.assert_allclose(se_OWN, rslt.bse[:, 2], rtol=1e-02,
+                                   atol=1e-02)
         np.testing.assert_allclose(t_OWN, rslt.tvalues[:, 2], rtol=1e-01,
                                    atol=1e-02)
         np.testing.assert_allclose(est_POP, rslt.params[:, 3], rtol=1e-03,
                                    atol=1e-02)
-        np.testing.assert_allclose(
-            se_POP, rslt.bse[:, 3], rtol=1e-02, atol=1e-02)
+        np.testing.assert_allclose(se_POP, rslt.bse[:, 3], rtol=1e-02,
+                                   atol=1e-02)
         np.testing.assert_allclose(t_POP, rslt.tvalues[:, 3], rtol=1e-01,
                                    atol=1e-02)
         np.testing.assert_allclose(est_UNEMP, rslt.params[:, 4], rtol=1e-04,
@@ -760,8 +787,8 @@ class TestGWRPoisson(unittest.TestCase):
 
 class TestGWRBinomial(unittest.TestCase):
     def setUp(self):
-        data_path = os.path.join(os.path.dirname(
-            __file__), 'clearwater/landslides.csv')
+        data_path = os.path.join(
+            os.path.dirname(__file__), 'clearwater/landslides.csv')
         data = io.open(data_path)
         self.coords = list(zip(data.by_col('X'), data.by_col('Y')))
         self.y = np.array(data.by_col('Landslid')).reshape((-1, 1))
@@ -772,14 +799,22 @@ class TestGWRBinomial(unittest.TestCase):
         SOUTH = np.array(data.by_col('AbsSouth')).reshape((-1, 1))
         DIST = np.array(data.by_col('DistStrm')).reshape((-1, 1))
         self.X = np.hstack([ELEV, SLOPE, SIN, COS, SOUTH, DIST])
-        self.BS_F = io.open(os.path.join(os.path.dirname(
-            __file__), 'clearwater/clearwater_BS_F_listwise.csv'))
-        self.BS_NN = io.open(os.path.join(os.path.dirname(
-            __file__), 'clearwater/clearwater_BS_NN_listwise.csv'))
-        self.GS_F = io.open(os.path.join(os.path.dirname(
-            __file__), 'clearwater/clearwater_GS_F_listwise.csv'))
-        self.GS_NN = io.open(os.path.join(os.path.dirname(
-            __file__), 'clearwater/clearwater_GS_NN_listwise.csv'))
+        self.BS_F = io.open(
+            os.path.join(
+                os.path.dirname(__file__),
+                'clearwater/clearwater_BS_F_listwise.csv'))
+        self.BS_NN = io.open(
+            os.path.join(
+                os.path.dirname(__file__),
+                'clearwater/clearwater_BS_NN_listwise.csv'))
+        self.GS_F = io.open(
+            os.path.join(
+                os.path.dirname(__file__),
+                'clearwater/clearwater_GS_F_listwise.csv'))
+        self.GS_NN = io.open(
+            os.path.join(
+                os.path.dirname(__file__),
+                'clearwater/clearwater_GS_NN_listwise.csv'))
 
     def test_BS_F(self):
         est_Int = self.BS_F.by_col(' est_Intercept')
@@ -806,8 +841,9 @@ class TestGWRBinomial(unittest.TestCase):
         yhat = self.BS_F.by_col(' yhat')
         pdev = np.array(self.BS_F.by_col(' localpdev')).reshape((-1, 1))
 
-        model = GWR(self.coords, self.y, self.X, bw=19642.170, family=Binomial(),
-                    kernel='bisquare', fixed=True, sigma2_v1=False)
+        model = GWR(self.coords, self.y, self.X, bw=19642.170,
+                    family=Binomial(), kernel='bisquare', fixed=True,
+                    sigma2_v1=False)
         rslt = model.fit()
 
         AICc = get_AICc(rslt)
@@ -876,10 +912,13 @@ class TestGWRBinomial(unittest.TestCase):
         AICc = get_AICc(rslt)
         AIC = get_AIC(rslt)
         BIC = get_BIC(rslt)
+        D2 = rslt.D2
 
         self.assertAlmostEquals(np.floor(AICc), 277.0)
         self.assertAlmostEquals(np.floor(AIC), 271.0)
         self.assertAlmostEquals(np.floor(BIC), 358.0)
+        self.assertAlmostEquals(np.round(D2, 3), 0.319)
+
         np.testing.assert_allclose(est_Int, rslt.params[:, 0], rtol=1e-00)
         np.testing.assert_allclose(se_Int, rslt.bse[:, 0], rtol=1e-00)
         np.testing.assert_allclose(t_Int, rslt.tvalues[:, 0], rtol=1e-00)
@@ -932,8 +971,9 @@ class TestGWRBinomial(unittest.TestCase):
         yhat = self.GS_F.by_col(' yhat')
         pdev = self.GS_F.by_col(' localpdev')
 
-        model = GWR(self.coords, self.y, self.X, bw=8929.061, family=Binomial(),
-                    kernel='gaussian', fixed=True, sigma2_v1=False)
+        model = GWR(self.coords, self.y, self.X, bw=8929.061,
+                    family=Binomial(), kernel='gaussian', fixed=True,
+                    sigma2_v1=False)
         rslt = model.fit()
 
         AICc = get_AICc(rslt)
