@@ -15,7 +15,11 @@ import sys
 
 from datetime import datetime, timedelta
 from subprocess import check_output
-from urllib2 import urlopen
+try:
+    from urllib import urlopen
+except:
+    from urllib.request import urlopen
+
 import ssl
 
 context = ssl._create_unverified_context()
@@ -48,7 +52,7 @@ def get_paged_request(url):
     results = []
     while url:
         print("fetching %s" % url, file=sys.stderr)
-        f = urlopen(url, context=context)
+        f = urlopen(url)
         results.extend(json.load(f))
         links = parse_link_header(f.headers)
         url = links.get('next')
